@@ -1,7 +1,9 @@
 package cmd
 
 import (
+	"fmt"
 	"log"
+	"time"
 
 	"github.com/beringresearch/macpine/host"
 	"github.com/beringresearch/macpine/qemu"
@@ -32,15 +34,23 @@ func shell(cmd *cobra.Command, args []string) {
 	}
 
 	machineConfig, err := qemu.GetMachineConfig(vmName)
-   if err != nil {
-      log.Fatalln(err)
-   }
+	if err != nil {
+		log.Fatalln(err)
+	}
 
 	if status, _ := machineConfig.Status(); status != "Running" {
 		log.Fatalf("%s is not running", machineConfig.Alias)
 	}
-	err = host.Exec(machineConfig, "bash")
-	if err != nil {
-		log.Fatalln(err)
+
+	for {
+		err = host.Exec(machineConfig, "bash")
+
+		if err == nil {
+			break
+		}
+
+		fmt.Print(".")
+		time.Sleep(4 * time.Second)
 	}
+
 }
